/* /////////////////////////////////////////////////////////////////////////////
 * File:        atlstl_support_error_info.h (formerly MAErInfo.h, ::SynesisAtl)
 *
 * Purpose:     SupportErrorInfoImpl class.
 *
 * Created:     17th April 1999
 * Updated:     30th May 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/atlstl
 *                          http://www.atlstl.org/
 *
 *              email:      submissions@atlstl.org  for submissions
 *                          admin@atlstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file atlstl_support_error_info.h
///
/// SupportErrorInfoImpl class.

#ifndef ATLSTL_INCL_H_ATLSTL_SUPPORT_ERROR_INFO
#define ATLSTL_INCL_H_ATLSTL_SUPPORT_ERROR_INFO

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
# define ATLSTL_VER_H_ATLSTL_SUPPORT_ERROR_INFO_MAJOR      2
# define ATLSTL_VER_H_ATLSTL_SUPPORT_ERROR_INFO_MINOR      1
# define ATLSTL_VER_H_ATLSTL_SUPPORT_ERROR_INFO_REVISION   6
# define ATLSTL_VER_H_ATLSTL_SUPPORT_ERROR_INFO_EDIT       47
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef ATLSTL_INCL_H_ATLSTL
# include "atlstl.h"    // Include the ATLSTL root header
#endif /* !ATLSTL_INCL_H_ATLSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The ATLSTL components are contained within the atlstl namespace. This is
 * actually an alias for stlsoft::atlstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _ATLSTL_NO_NAMESPACE    atlstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::atlstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             atlstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _ATLSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::atlstl */
namespace atlstl
{
# else
/* Define stlsoft::atlstl_project */

namespace stlsoft
{

namespace atlstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_ATLSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \defgroup atlstl_interface_classes Interface Classes
/// \ingroup ATLSTL
/// \brief Interface (template) classes
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Provides implementation of ISupportErrorInfo for support for errors on one
/// interface
template <const IID *piid>
class ATL_NO_VTABLE SupportErrorInfoImpl
    : public ISupportErrorInfo
{
public:
    typedef SupportErrorInfoImpl<piid>  Class;

// ISupportErrorInfo
public:
    STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
    {
        return (InlineIsEqualGUID(riid, *piid))
                    ? S_OK
                    : S_FALSE;
    }
};


/// Provides implementation of ISupportErrorInfo for support for errors on two
/// interfaces
template <const IID *piid1, const IID *piid2>
class ATL_NO_VTABLE SupportErrorInfoImpl2
    : public ISupportErrorInfo
{
public:
    typedef SupportErrorInfoImpl2<piid1, piid2> Class;

// ISupportErrorInfo
public:
    STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
    {
        return (InlineIsEqualGUID(riid, *piid1) ||
                InlineIsEqualGUID(riid, *piid2))
                    ? S_OK
                    : S_FALSE;
    }
};


/// Provides implementation of ISupportErrorInfo for support for errors on three
/// interfaces
template <const IID *piid1, const IID *piid2, const IID *piid3>
class ATL_NO_VTABLE SupportErrorInfoImpl3
    : public ISupportErrorInfo
{
public:
    typedef SupportErrorInfoImpl3<piid1, piid2, piid3>  Class;

// ISupportErrorInfo
public:
    STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
    {
        return (InlineIsEqualGUID(riid, *piid1) ||
                InlineIsEqualGUID(riid, *piid2) ||
                InlineIsEqualGUID(riid, *piid3))
                    ? S_OK
                    : S_FALSE;
    }
};

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group atlstl_interface_classes

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _ATLSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace atlstl
# else
} // namespace atlstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_ATLSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !ATLSTL_INCL_H_ATLSTL_SUPPORT_ERROR_INFO */

/* ////////////////////////////////////////////////////////////////////////// */
