/* ////////////////////////////////////////////////////////////////////////////
 * File:        comstl_errorinfo_functions.h
 *
 * Purpose:     Error info functions.
 *
 * Created:
 * Updated:     30th May 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/comstl
 *                          http://www.comstl.org/
 *
 *              email:      submissions@comstl.org  for submissions
 *                          admin@comstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file comstl_errorinfo_functions.h
///
/// Error info functions.

#ifndef COMSTL_INCL_H_COMSTL_ERRORINFO_FUNCTIONS
#define COMSTL_INCL_H_COMSTL_ERRORINFO_FUNCTIONS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
# define COMSTL_VER_H_COMSTL_ERRORINFO_FUNCTIONS_MAJOR      1
# define COMSTL_VER_H_COMSTL_ERRORINFO_FUNCTIONS_MINOR      2
# define COMSTL_VER_H_COMSTL_ERRORINFO_FUNCTIONS_REVISION   3
# define COMSTL_VER_H_COMSTL_ERRORINFO_FUNCTIONS_EDIT       9
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef COMSTL_INCL_H_COMSTL
# include "comstl.h"    // Include the COMSTL root header
#endif /* !COMSTL_INCL_H_COMSTL */
#ifdef STLSOFT_UNITTEST
# include <wchar.h>
#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The COMSTL components are contained within the comstl namespace. This is
 * actually an alias for stlsoft::comstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _COMSTL_NO_NAMESPACE    comstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::comstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             comstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _COMSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::comstl */
namespace comstl
{
# else
/* Define stlsoft::comstl_project */

namespace stlsoft
{

namespace comstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup comstl_error_library Error Library
/// \ingroup COMSTL libraries
/// \brief This library provides facilities for manipulating COM error information
/// @{

/* ////////////////////////////////////////////////////////////////////////////
 * Functions
 */

inline HRESULT set_error_info(LPCOLESTR desc)
{
    ICreateErrorInfo    *pCEI;
    HRESULT             hr = ::CreateErrorInfo(&pCEI);

    if(SUCCEEDED(hr))
    {
        IErrorInfo  *pEI;

        // Ask for IErrorInfo
        hr = pCEI->QueryInterface(IID_IErrorInfo, (void**)&pEI);

        if(SUCCEEDED(hr))
        {
            hr = pCEI->SetDescription(const_cast<LPOLESTR>(desc));

            ::SetErrorInfo(0, pEI);

            pEI->Release();
        }

        pCEI->Release();
    }

    return hr;
}

////////////////////////////////////////////////////////////////////////////////
// Unit-testing

#ifdef STLSOFT_UNITTEST

namespace unittest
{
    ss_bool_t test_comstl_errorinfo_functions(unittest_reporter *r)
    {
        using stlsoft::unittest::unittest_initialiser;

        ss_bool_t               bSuccess    =   true;

        unittest_initialiser    init(r, "COMSTL", "errorinfo_functions", __FILE__);

        set_error_info(L"Test info #1");

        IErrorInfo  *pei;
        HRESULT     hr  =   ::GetErrorInfo(0, &pei);

        if(SUCCEEDED(hr))
        {
            BSTR    bstrDesc;

            hr = pei->GetDescription(&bstrDesc);

            if(SUCCEEDED(hr))
            {
                if(0 != wcscmp(bstrDesc, L"Test info #1"))
                {
                    r->report("Error information not as expected ", __LINE__);
                    bSuccess = false;
                }

                ::SysFreeString(bstrDesc);
            }

            pei->Release();
        }

        return bSuccess;
    }

    unittest_registrar    unittest_comstl_errorinfo_functions(test_comstl_errorinfo_functions);

} // namespace unittest

#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group comstl_error_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _COMSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace comstl
# else
} // namespace comstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !COMSTL_INCL_H_COMSTL_ERRORINFO_FUNCTIONS */

/* ////////////////////////////////////////////////////////////////////////// */
