/* /////////////////////////////////////////////////////////////////////////////
 * File:        dotnetstl_dispose_functions.h
 *
 * Purpose:     A useful tool for accessing a String object's content as a c-string.
 *
 * Created:     24th June 2003
 * Updated:     30th May 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/dotnetstl
 *                          http://www.dotnetstl.org/
 *
 *              email:      submissions@dotnetstl.org  for submissions
 *                          admin@dotnetstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file dotnetstl_dispose_functions.h
///
/// A useful tool for accessing a String object's content as a c-string.

#ifndef DOTNETSTL_INCL_H_DOTNETSTL_DISPOSE_FUNCTIONS
#define DOTNETSTL_INCL_H_DOTNETSTL_DISPOSE_FUNCTIONS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
# define DOTNETSTL_VER_H_DOTNETSTL_DISPOSE_FUNCTIONS_MAJOR    1
# define DOTNETSTL_VER_H_DOTNETSTL_DISPOSE_FUNCTIONS_MINOR    2
# define DOTNETSTL_VER_H_DOTNETSTL_DISPOSE_FUNCTIONS_REVISION 4
# define DOTNETSTL_VER_H_DOTNETSTL_DISPOSE_FUNCTIONS_EDIT     11
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef DOTNETSTL_INCL_H_DOTNETSTL
# include "dotnetstl.h"                     // Include the .netSTL root header
#endif /* !DOTNETSTL_INCL_H_DOTNETSTL */

/* ////////////////////////////////////////////////////////////////////////// */

#ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::dotnetstl */
namespace dotnetstl
{
#else
/* Define stlsoft::dotnet_project */

namespace stlsoft
{

namespace dotnetstl_project
{

#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Disposes the managed type, and resets the pointer
///
/// \param pt The pointer on which to call Dispose(). Can be null, but otherwise must be convertible to IDispose
template <ss_typename_param_k T>
inline void dispose_set_null(T *&pt)
{
    if(0 != pt)
    {
        System::IDisposable *disposable =   pt;

        disposable->Dispose();
        pt = 0;
    }
}

/// Disposes all the items in a container
///
/// \param pc The container on whose items to call Dispose().
template <ss_typename_param_k C>
inline void dispose_contents(C *pc)
{
    for(int i = 0, count = pc->get_Count(); i < count; ++i)
    {
        System::IDisposable *o  =   dynamic_cast<System::IDisposable*>(pc->get_Item(i));

        dispose_set_null(o);
    }
}

////////////////////////////////////////////////////////////////////////////////
// Unit-testing

#ifdef STLSOFT_UNITTEST

namespace unittest
{
    ss_bool_t test_dotnetstl_dispose_functions(unittest_reporter *r)
    {
        using stlsoft::unittest::unittest_initialiser;

        ss_bool_t               bSuccess    =   true;

        unittest_initialiser    init(r, ".netSTL", "dispose_functions", __FILE__);

#if 0
        if(<<TODO>>)
        {
            r->report("<<TODO>> failed ", __LINE__);
            bSuccess = false;
        }
#endif /* 0 */

        return bSuccess;
    }

    unittest_registrar    unittest_dotnetstl_dispose_functions(test_dotnetstl_dispose_functions);

} // namespace unittest

#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

#ifdef _STLSOFT_NO_NAMESPACE
} // namespace dotnetstl
#else
} // namespace dotnetstl_project
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* DOTNETSTL_INCL_H_DOTNETSTL_DISPOSE_FUNCTIONS */

/* ////////////////////////////////////////////////////////////////////////// */
