/* /////////////////////////////////////////////////////////////////////////////
 * File:        mfcstl_window_access.h
 *
 * Purpose:     Contains access shims for windows.
 *
 * Created:     10th January 2003
 * Updated:     30th May 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/mfcstl
 *                          http://www.mfcstl.org/
 *
 *              email:      submissions@mfcstl.org  for submissions
 *                          admin@mfcstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file mfcstl_window_access.h
///
/// Contains access shims for windows.

#ifndef MFCSTL_INCL_H_MFCSTL_WINDOW_ACCESS
#define MFCSTL_INCL_H_MFCSTL_WINDOW_ACCESS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
# define MFCSTL_VER_H_MFCSTL_WINDOW_ACCESS_MAJOR     1
# define MFCSTL_VER_H_MFCSTL_WINDOW_ACCESS_MINOR     3
# define MFCSTL_VER_H_MFCSTL_WINDOW_ACCESS_REVISION  3
# define MFCSTL_VER_H_MFCSTL_WINDOW_ACCESS_EDIT      17
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef MFCSTL_INCL_H_MFCSTL
# include "mfcstl.h"                // Include the MFCSTL root header
#endif /* !MFCSTL_INCL_H_MFCSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _MFCSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::mfcstl */
namespace mfcstl
{
# else
/* Define stlsoft::mfcstl_project */

namespace stlsoft
{

namespace mfcstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_MFCSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup concepts STLSoft Concepts

/// \weakgroup concepts_shims Shims
/// \ingroup concepts

/// \weakgroup concepts_shims_window_access Window Access Shims
/// \ingroup concepts_shims
/// \brief These \ref concepts_shims "shims" retrieve the HWND for arbitrary types

/// \defgroup mfcstl_window_access_shims Window Access Shims (MFCSTL)
/// \ingroup MFCSTL concepts_shims_window_access
/// \brief These \ref concepts_shims "shims" retrieve the HWND for arbitrary types
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * get_hwnd
 *
 * This can be applied to an expression, and the return value is the
 * corresponding HWND.
 */

/* HWND */

/// \brief Access the HWND of the given CWnd instance
///
/// This access <a href = "http://stlsoft.org/white_papers.html#shims">shim</a> retrieves the
/// HWND window handle for the given CWnd instance.
///
/// \param w A reference to the CWnd instance whose HWND will be retrieved
/// \return The HWND corresponding to the given CWnd instance \c w
inline HWND get_hwnd(CWnd const &w)
{
    return w.m_hWnd;
}

////////////////////////////////////////////////////////////////////////////////
// Unit-testing

#ifdef STLSOFT_UNITTEST

namespace unittest
{
    ss_bool_t test_mfcstl_window_access(unittest_reporter *r)
    {
        using stlsoft::unittest::unittest_initialiser;

        ss_bool_t               bSuccess    =   true;

        unittest_initialiser    init(r, "MFCSTL", "window_access", __FILE__);

        HWND    hwndDesktop =   ::GetDesktopWindow();
        CWnd    wndDesktop;

        wndDesktop.Attach(hwndDesktop);

        if(get_hwnd(wndDesktop) != hwndDesktop)
        {
            r->report("get_hwnd(CWindow) failed ", __LINE__);
            bSuccess = false;
        }

        wndDesktop.Detach();

        return bSuccess;
    }

    unittest_registrar    unittest_mfcstl_window_access(test_mfcstl_window_access);

} // namespace unittest

#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group mfcstl_window_access_shims

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _MFCSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace mfcstl
# else
} // namespace mfcstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_MFCSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* MFCSTL_INCL_H_MFCSTL_SYSTEM_INFO */

/* ////////////////////////////////////////////////////////////////////////// */
