/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_sap_cast.h
 *
 * Purpose:     sap_cast - a dangerous weapon in the hands of the unwary.
 *
 * Created:     25th February 2004
 * Updated:     30th May 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file stlsoft_sap_cast.h
///
/// sap_cast - a dangerous weapon in the hands of the unwary.

#ifndef STLSOFT_INCL_H_STLSOFT_SAP_CAST
#define STLSOFT_INCL_H_STLSOFT_SAP_CAST

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
# define STLSOFT_VER_H_STLSOFT_SAP_CAST_MAJOR       1
# define STLSOFT_VER_H_STLSOFT_SAP_CAST_MINOR       3
# define STLSOFT_VER_H_STLSOFT_SAP_CAST_REVISION    2
# define STLSOFT_VER_H_STLSOFT_SAP_CAST_EDIT        8
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef STLSOFT_INCL_H_STLSOFT
# include "stlsoft.h"               // Include the STLSoft root header
#endif /* !STLSOFT_INCL_H_STLSOFT */
#ifndef STLSOFT_INCL_H_CONSTRAINTS
# include "stlsoft_constraints.h"   // stlsoft_constraint_must_be_pod
#endif /* !STLSOFT_INCL_H_CONSTRAINTS */
#ifndef STLSOFT_INCL_H_TYPE_TRAITS
# include "stlsoft_type_traits.h"   //
#endif /* !STLSOFT_INCL_H_TYPE_TRAITS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

template<   ss_typename_param_k TO
        ,   ss_typename_param_k FROM
        >
inline TO sap_cast(FROM from)
{
#if defined(__STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT) && \
    (   !defined(__STLSOFT_COMPILER_IS_BORLAND) || \
        __BORLANDC__ >= 0x0564)
    // Both types must be pointer types
    stlsoft_static_assert(0 != base_type_traits<FROM>::is_pointer);
    stlsoft_static_assert(0 != base_type_traits<TO>::is_pointer);

    typedef ss_typename_type_k base_type_traits<FROM>::base_type    from_base_type;
    typedef ss_typename_type_k base_type_traits<TO>::base_type      to_base_type;

    // The intermediate type might be void *, void const *, void volatile * or
    // void const volatile *
    typedef ss_typename_type_k select_first_type<   void const *
                                                ,   void *
                                                ,   base_type_traits<FROM>::is_const
                                                >::type     non_volatile_type;
    typedef ss_typename_type_k select_first_type<   void const volatile *
                                                ,   void volatile *
                                                ,   base_type_traits<FROM>::is_const
                                                >::type     volatile_type;
    typedef ss_typename_type_k select_first_type<   volatile_type
                                                ,   non_volatile_type
                                                ,   base_type_traits<FROM>::is_volatile
                                                >::type     pointer_type;

    // "static_cast" to void (const) (volatile) *
    pointer_type        pv  =   from;
#else /* __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT */
    void const volatile *p1 =   from;
    void                *pv =   const_cast<void*>(p1);
#endif /* __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT */

    // static_cast to destination type
    return static_cast<TO>(pv);
}

////////////////////////////////////////////////////////////////////////////////
// Unit-testing

#ifdef STLSOFT_UNITTEST

namespace unittest
{
    ss_bool_t test_stlsoft_sap_cast(unittest_reporter *r)
    {
        ss_bool_t               bSuccess    =   true;

        unittest_initialiser    init(r, "STLSoft", "sap_cast", __FILE__);

        int     i;
        void    *pv =   &i;
        short   *ps =   reinterpret_cast<short*>(&i);

        if(sap_cast<short*>(pv) != ps)
        {
            r->report("sap_cast failed ", __LINE__);
            bSuccess = false;
        }

        return bSuccess;
    }

    unittest_registrar    unittest_stlsoft_sap_cast(test_stlsoft_sap_cast);

} // namespace unittest

#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !STLSOFT_INCL_H_STLSOFT_SAP_CAST */

/* ////////////////////////////////////////////////////////////////////////// */
