/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_functions.h (formerly MWBase.h, ::SynesisWin)
 *
 * Purpose:     A number of useful functions .
 *
 * Created:     7th May 2000
 * Updated:     30th May 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2000-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file winstl_functions.h
///
/// A number of useful functions .

#ifndef WINSTL_INCL_H_WINSTL_FUNCTIONS
#define WINSTL_INCL_H_WINSTL_FUNCTIONS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
# define WINSTL_VER_H_WINSTL_FUNCTIONS_MAJOR        1
# define WINSTL_VER_H_WINSTL_FUNCTIONS_MINOR        9
# define WINSTL_VER_H_WINSTL_FUNCTIONS_REVISION     4
# define WINSTL_VER_H_WINSTL_FUNCTIONS_EDIT         19
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef WINSTL_INCL_H_WINSTL
# include "winstl.h"                // Include the WinSTL root header
#endif /* !WINSTL_INCL_H_WINSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
# else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

#ifdef __cplusplus
inline bool BOOL2bool(BOOL b)
{
    return b != 0;
}
inline BOOL bool2BOOL(bool b)
{
    return b != false;
}
#else
# define BOOL2bool(b)               SyCastStatic(bool, ((b) != 0))
# define bool2BOOL(b)               SyCastStatic(BOOL, ((b) != false))
#endif /* __cplusplus */


/** CloseHandleSetNull
 *
 * Closes a handle, and sets its value to NULL
 */

inline ws_bool_t CloseHandleSetNull(HANDLE &h)
{
    return ::CloseHandle(h) ? (h = NULL, true) : false;
}

/** DuplicateLocalHandle()
 *
 * Duplicates a handle. Both the handle and the duplicate are within the context
 * of the current process.
 */
inline ws_bool_t DuplicateLocalHandle(HANDLE h, HANDLE &hDup)
{
    return BOOL2bool(::DuplicateHandle(::GetCurrentProcess(), h, ::GetCurrentProcess(), &hDup, 0, false, DUPLICATE_SAME_ACCESS));
}

/** FormatMessage()
 *
 */

template <ss_typename_param_k C>
inline C *fmt_msg_find_last_good__(C *first, C* last)
{
    for(;first != last; )
    {
        switch(*(last - 1))
        {
            case    ' ':
            case    '\t':
            case    '\r':
            case    '\n':
            case    '.':
                *(last - 1) = '\0';
                --last;
                break;
            default:
                first = last;
                break;
        }
    }

    return last;
}

inline ws_dword_t FormatMessage(DWORD error, HINSTANCE hinst, ws_char_a_t *buffer, ws_uint_t cchBuffer)
{
    ws_dword_t  flags   =   FORMAT_MESSAGE_MAX_WIDTH_MASK | (NULL == hinst ? FORMAT_MESSAGE_FROM_SYSTEM : FORMAT_MESSAGE_FROM_HMODULE);
    ws_dword_t  res     = ::FormatMessageA(flags, static_cast<void*>(hinst), error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), buffer, cchBuffer, NULL);

    if(res != 0)
    {
        // Now trim the trailing space
        ws_char_a_t *last_good = fmt_msg_find_last_good__(buffer, buffer + res);

        *last_good = 0;
        res = static_cast<ws_dword_t>(last_good - buffer);

        winstl_assert(res >= 0);
    }

    return res;
}

inline ws_dword_t FormatMessage(DWORD error, ws_char_a_t *buffer, ws_uint_t cchBuffer)
{
    return FormatMessage(error, NULL, buffer, cchBuffer);
}

inline ws_dword_t FormatMessage(DWORD error, ws_char_w_t *buffer, ws_uint_t cchBuffer)
{
    ws_dword_t res = ::FormatMessageW(  FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_MAX_WIDTH_MASK,
                                        NULL,
                                        error,
                                        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                                        buffer,
                                        cchBuffer,
                                        NULL);

    if(res != 0)
    {
        // Now trim the trailing space
        ws_char_w_t *last_good = fmt_msg_find_last_good__(buffer, buffer + res);

        *last_good = 0;
        res = static_cast<ws_dword_t>(last_good - buffer);

        winstl_assert(res >= 0);
    }

    return res;
}

inline ws_dword_t FormatMessage(DWORD error, HINSTANCE hinstSource, ws_char_a_t **buffer)
{
    ws_dword_t  flags   =   FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_MAX_WIDTH_MASK | (NULL == hinstSource ? FORMAT_MESSAGE_FROM_SYSTEM : FORMAT_MESSAGE_FROM_HMODULE);
    ws_dword_t  res     =   ::FormatMessageA(flags, (void*)hinstSource, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPSTR)buffer, 0, NULL);

    if(res != 0)
    {
        // Now trim the trailing space
        ws_char_a_t *last_good = fmt_msg_find_last_good__(*buffer, *buffer + res);

        *last_good = 0;
        res = static_cast<ws_dword_t>(last_good - *buffer);

        winstl_assert(res >= 0);
    }

    return res;
}

inline ws_dword_t FormatMessage(DWORD error, HINSTANCE hinstSource, ws_char_w_t **buffer)
{
    ws_dword_t  flags   =   FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_MAX_WIDTH_MASK | (NULL == hinstSource ? FORMAT_MESSAGE_FROM_SYSTEM : FORMAT_MESSAGE_FROM_HMODULE);
    ws_dword_t  res     =   ::FormatMessageW(flags, (void*)hinstSource, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPWSTR)buffer, 0, NULL);

    if(res != 0)
    {
        // Now trim the trailing space
        ws_char_w_t *last_good = fmt_msg_find_last_good__(*buffer, *buffer + res);

        *last_good = 0;
        res = static_cast<ws_dword_t>(last_good - *buffer);

        winstl_assert(res >= 0);
    }

    return res;
}



/** Elicits asynchronous key state
 */
inline ws_bool_t IsKeyPressedAsync(int k)
{
    return (::GetAsyncKeyState(k) & 0x8000) != 0;
}

/** Gets the style of the window */
inline ws_long_t GetStyle(HWND h)
{
    return GetWindowLong(h, GWL_STYLE);
}

/** Gets the extended style of the window */
inline ws_long_t GetExStyle(HWND h)
{
    return GetWindowLong(h, GWL_EXSTYLE);
}

/** Sets the style of the window */
inline ws_long_t SetStyle(HWND h, ws_long_t s)
{
    return SetWindowLong(h, GWL_STYLE, (ws_long_t)s);
}

/** Sets the extended style of the window */
inline ws_long_t SetExStyle(HWND h, ws_long_t x)
{
    return SetWindowLong(h, GWL_EXSTYLE, (ws_long_t)x);
}

/** Modifies the style of the window */
inline ws_long_t ModifyStyle(HWND h, ws_long_t sRem, ws_long_t sAdd)
{
    return SetStyle(h, (GetStyle(h) & ~sRem) | sAdd);
}

/** Modifies the extended style of the window */
inline ws_long_t ModifyExStyle(HWND h, ws_long_t sRem, ws_long_t sAdd)
{
    return SetExStyle(h, (GetExStyle(h) & ~sRem) | sAdd);
}

/** Tests whether the given window has the given window class */
inline ws_bool_t IsWindowClass(HWND hwnd, ws_char_a_t const *name)
{
    winstl_assert(NULL != hwnd);
    winstl_assert(NULL != name);

    ws_char_a_t szName[256];

    (void)::GetClassNameA(hwnd, szName, stlsoft_num_elements(szName));

    return 0 == lstrcmpiA(szName, name);
}

/** Tests whether the given window has the given window class */
inline ws_bool_t IsWindowClass(HWND hwnd, ws_char_w_t const *name)
{
    winstl_assert(NULL != hwnd);
    winstl_assert(NULL != name);

    ws_char_w_t szName[256];

    (void)::GetClassNameW(hwnd, szName, stlsoft_num_elements(szName));

    return 0 == lstrcmpiW(szName, name);
}

/** Enables/disable a dialog item
 */
inline void EnableDlgItem(HWND hwnd, int id, ws_bool_t bEnable)
{
    ::EnableWindow(::GetDlgItem(hwnd, id), bEnable);
}

/** Elicits the enable status of a dialog item
 */
inline ws_bool_t IsDlgItemEnabled(HWND hwnd, int id)
{
    return ::IsWindowEnabled(::GetDlgItem(hwnd, id)) != 0;
}

/** Gets the text length of a dialog item's window contents
 */
inline int GetDlgItemTextLength(HWND hwnd, int id)
{
    return ::GetWindowTextLength(::GetDlgItem(hwnd, id));
}

/** Gets the HINSTANCE associated with a given window
 */
#ifdef GetWindowInstance
# undef GetWindowInstance
#endif /* GetWindowInstance */
inline HINSTANCE GetWindowInstance(HWND hwnd)
{
    return reinterpret_cast<HINSTANCE>(::GetWindowLong(hwnd, GWL_HINSTANCE));
}

////////////////////////////////////////////////////////////////////////////////
// Unit-testing

#ifdef STLSOFT_UNITTEST

namespace unittest
{
    ss_bool_t test_winstl_functions(unittest_reporter *r)
    {
        using stlsoft::unittest::unittest_initialiser;

        ss_bool_t               bSuccess    =   true;

        unittest_initialiser    init(r, "WinSTL", "functions", __FILE__);

#if 0
        if(<<TODO>>)
        {
            r->report("<<TODO>> failed ", __LINE__);
            bSuccess = false;
        }
#endif /* 0 */

        return bSuccess;
    }

    unittest_registrar    unittest_winstl_functions(test_winstl_functions);

} // namespace unittest

#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace winstl
# else
} // namespace winstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* WINSTL_INCL_H_WINSTL_FUNCTIONS */

/* ////////////////////////////////////////////////////////////////////////// */
