/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_windows_type_conversions.h (originally MWWinCmn.h, ::SynesisWin)
 *
 * Purpose:     Type conversions for Windows.
 *
 * Created:
 * Updated:     30th May 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file winstl_windows_type_conversions.h
///
/// Type conversions for Windows.

#ifndef WINSTL_INCL_H_WINSTL_WINDOWS_TYPE_CONVERSIONS
#define WINSTL_INCL_H_WINSTL_WINDOWS_TYPE_CONVERSIONS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
# define WINSTL_VER_H_WINSTL_WINDOWS_TYPE_CONVERSIONS_MAJOR       1
# define WINSTL_VER_H_WINSTL_WINDOWS_TYPE_CONVERSIONS_MINOR       2
# define WINSTL_VER_H_WINSTL_WINDOWS_TYPE_CONVERSIONS_REVISION    5
# define WINSTL_VER_H_WINSTL_WINDOWS_TYPE_CONVERSIONS_EDIT        16
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef WINSTL_INCL_H_WINSTL
# include "winstl.h"                // Include the WinSTL root header
#endif /* !WINSTL_INCL_H_WINSTL */
#ifndef STLSOFT_INCL_H_STLSOFT_UNION_CAST
# include "stlsoft_union_cast.h"    // stlsoft::union_cast
#endif /* !STLSOFT_INCL_H_STLSOFT_UNION_CAST */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
# else
/* Define stlsoft::winstl_project */
namespace stlsoft
{
namespace winstl_project
{
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

typedef stlsoft_ns_qual(union_cast)<LPARAM, char const *>       StrA2LPARAM;
typedef stlsoft_ns_qual(union_cast)<LPARAM, wchar_t const *>    StrW2LPARAM;
typedef stlsoft_ns_qual(union_cast)<LPARAM, TCHAR const *>      StrT2LPARAM;

typedef stlsoft_ns_qual(union_cast)<HWND, WPARAM>               WPARAM2HWND;
typedef stlsoft_ns_qual(union_cast)<HWND, LPARAM>               LPARAM2HWND;

typedef stlsoft_ns_qual(union_cast)<WPARAM, HWND>               HWND2WPARAM;
typedef stlsoft_ns_qual(union_cast)<LPARAM, HWND>               HWND2LPARAM;
typedef stlsoft_ns_qual(union_cast)<UINT, HWND>                 HWND2UINT;

typedef stlsoft_ns_qual(union_cast)<int, WPARAM>                WPARAM2int;

typedef stlsoft_ns_qual(union_cast)<HDROP, WPARAM>              WPARAM2HDROP;

typedef stlsoft_ns_qual(union_cast)<WPARAM, void*>              Ptr2WPARAM;
typedef stlsoft_ns_qual(union_cast)<LPARAM, void*>              Ptr2LPARAM;
typedef stlsoft_ns_qual(union_cast)<UINT, void*>                Ptr2UINT;

typedef stlsoft_ns_qual(union_cast)<char const *, int>          Id2StrA;
typedef stlsoft_ns_qual(union_cast)<wchar_t const *, int>       Id2StrW;
typedef stlsoft_ns_qual(union_cast)<TCHAR const *, int>         Id2StrT;

#if defined(_INC_COMMCTRL) || \
    defined(_COMMCTRL_H)
 typedef union_cast<LPNMHDR, LPARAM>                            LPARAM2LPNMHDR;
 typedef union_cast<HTREEITEM, LONG>                            LONG2HTREEITEM;
#endif /* _INC_COMMCTRL || _COMMCTRL_H */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(__STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace winstl
# else
} // namespace winstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* WINSTL_INCL_H_WINSTL_WINDOWS_TYPE_CONVERSIONS */

/* ////////////////////////////////////////////////////////////////////////// */
